@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2006-2019 by Ilya Kotov                                 *
+ *   Copyright (C) 2006-2021 by Ilya Kotov                                 *
  *   forkotov02@ya.ru                                                      *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -47,14 +47,6 @@ class PlayListTask;
  */
 struct SimpleSelection
 {
-    /*!
-     * Constructs an empty selection model.
-     */
-    SimpleSelection()
-    {
-        m_bottom = -1;
-        m_top = 1;
-    }
     /*!
      * Returns \p true if this selection is valid; otherwise returns returns \p false.
      */
@@ -76,8 +68,8 @@ struct SimpleSelection
     {
         return m_bottom - m_top + 1;
     }
-    int m_bottom;               /*!< Bottom of the selection */
-    int m_top;                  /*!< Top of the selection   */
+    int m_bottom = -1;              /*!< Bottom of the selection */
+    int m_top = 1;                  /*!< Top of the selection   */
     QList<int> m_selected_indexes;  /*!< Selected items numbers */
 };
 /*! @brief The PlayListModel class provides a data model for the playlist.
@@ -109,7 +101,7 @@ public:
      * @param name Playlist name.
      * @param parent QObject parent
      */
-    PlayListModel(const QString &name, QObject *parent = nullptr);
+    explicit PlayListModel(const QString &name, QObject *parent = nullptr);
     /*!
      * Object destructor.
      */
@@ -202,13 +194,13 @@ public:
      * @param tracks List of tracks.
      * @param selected Selection state (\b true - select, \b false - unselect)
      */
-    void setSelected(QList<PlayListTrack *> tracks, bool selected = true);
+    void setSelected(const QList<PlayListTrack *> &tracks, bool selected = true);
     /*!
      * Sets the selected state of the list of items to \b select
      * @param items List of items.
      * @param selected Selection state (\b true - select, \b false - unselect)
      */
-    void setSelected(QList<PlayListItem *> items, bool selected = true);
+    void setSelected(const QList<PlayListItem *> &items, bool selected = true);
     /*!
      * Sets the selected state of the list of items range.
      * @param first Firts item in the range.
@@ -221,7 +213,7 @@ public:
      * @param indexes List of item \b indexes.
      * @param selected Selection state (\b true - select, \b false - unselect)
      */
-    void setSelected(QList<int> indexes, bool selected = true);
+    void setSelected(const QList<int> &indexes, bool selected = true);
     /*!
      * Advances to the next item. Returns \b false if next iten doesn't exist,
      * otherwise returns \b true
@@ -245,7 +237,7 @@ public:
     /*!
      * Returns \b true if \b f file is in play queue, otherwise returns \b false.
      */
-    bool isQueued(PlayListTrack* item) const;
+    bool isQueued(PlayListTrack *item) const;
     /*!
      * Returns \b true if play queue is empty,otherwise returns - \b false.
      */
@@ -253,7 +245,7 @@ public:
     /*!
      * Returns index of \b f file in queue.e
      */
-    int queuedIndex(PlayListTrack* track) const;
+    int queuedIndex(PlayListTrack *track) const;
     /*!
      * Returns the number of items in the queue
      */
@@ -261,7 +253,7 @@ public:
     /*!
      * Returns \b true if playback stops after \b item, otherwise returns \b false.
      */
-    bool isStopAfter(PlayListItem* item) const;
+    bool isStopAfter(const PlayListItem* item) const;
     /*!
      * Returns current selection(playlist can contain a lot of selections,
      * this method returns selection which \b row belongs to)
@@ -290,7 +282,7 @@ public:
     /*!
      * Returns total duration in milliseconds of all songs.
      */
-    int totalDuration() const;
+    qint64 totalDuration() const;
     /*!
      * Loads playlist with \b f_name name.
      */
@@ -399,7 +391,7 @@ public slots:
      * Adds a list of tracks to the playlist.
      * @param tracks List of tracks.
      */
-    void add(QList <PlayListTrack *> tracks);
+    void add(const QList<PlayListTrack *> &tracks);
     /*!
      * Adds a list of files and directories to the playlist
      * @param path Full path of file or directory.
@@ -421,11 +413,16 @@ public slots:
     /*!
      * Inserts \b tracks at index position \b index in the playlist.
      */
-    void insert(int index, QList <PlayListTrack *> tracks);
+    void insert(int index, const QList<PlayListTrack *> &tracks);
+    /*!
+     * Inserts serialized content \b json at index position \b index in the playlist.
+     * May be useful for fast drag-and-drop.
+     */
+    void insert(int index, const QByteArray &json);
     /*!
      * Inserts \b tracks after item \b before in the playlist.
      */
-    void insert(PlayListItem *before, QList <PlayListTrack *> tracks);
+    void insert(PlayListItem *before, const QList<PlayListTrack *> &tracks);
     /*!
      * Inserts file or directory at index position \b index in the playlist.
      * @param path Full path of file or directory.
@@ -593,14 +590,16 @@ private slots:
 
     void onTaskFinished();
 
+    void updateMetaData(const QStringList &paths);
+
 private:
-    PlayListTrack* m_current_track;
-    PlayListTrack* m_stop_track;
-    int m_current;
+    PlayListTrack *m_current_track = nullptr;
+    PlayListTrack *m_stop_track = nullptr;
+    int m_current = 0;
     SimpleSelection m_selection;  /*!< This flyweight object represents current selection. */
     QQueue <PlayListTrack*> m_queued_songs; /*!< Songs in play queue. */
     PlayState* m_play_state; /*!< Current playing state (Normal or Shuffle) */
-    qint64 m_total_duration;
+    qint64 m_total_duration = 0;
     FileLoader *m_loader;
     QString m_name;
     PlayListContainer *m_container;
